@@ -6,7 +6,7 @@ Convos - Multiuser IRC proxy with web interface
 
 =head1 VERSION
 
-0.5
+0.6
 
 =head1 DESCRIPTION
 
@@ -102,11 +102,6 @@ Set CONVOS_DEBUG for extra debug output to STDERR.
 
 Disable the frontend from automatically starting the backend.
 
-=item * CONVOS_PING_INTERVAL=30
-
-Set how often to send "keep-alive" through the web socket. Default is
-every 30 second.
-
 =item * CONVOS_REDIS_URL
 
 This is the URL to the Redis backend, and should follow this format:
@@ -215,7 +210,7 @@ use Convos::Core;
 use Convos::Core::Util ();
 use Convos::Upgrader;
 
-our $VERSION = '0.5';
+our $VERSION = '0.6';
 
 =head1 ATTRIBUTES
 
@@ -288,6 +283,7 @@ sub startup {
     delete $self->log->{handle};           # make sure it's fresh to file
   }
 
+  $self->ua->max_redirects(2);             # support getting facebook pictures
   $self->cache;                            # make sure cache is ok
   $self->plugin('Convos::Plugin::Helpers');
   $self->secrets([time]);                  # will be replaced by _set_secrets()
